const db = require('../db');
const authwrap = require('../authwrap');

const create_thread = async (msg, respond, socket) => {
	// validate inputs
	if (typeof msg.name !== 'string') {
		return respond({
			success: false,
			message: 'thread name required'
		});
	}
	if (msg.name.length > 200) {
		return respond({
			success: false,
			message: 'thread name 200 chars max'
		});
	}
	// add to db
	const insert = await db.query(
		'insert into thread (name, creator) values (?, ?) returning id',
		[msg.name, msg.auth_user.id]
	);
	const thread_id = insert.rows[0].id;
	// set up permissions
	if (!msg.permissions || !msg.permissions.view_limited) {
		await db.query(
			`insert into permission (thread, type, mutable, permission, value) 
			values (?, ?, ?, ?, ?)`,
			[thread_id, 'everyone', false, 'view', 'true']
		);
		if (!msg.permissions || !msg.permissions.post_limited) {
			await db.query(
				`insert into permission (thread, type, mutable, permission, value) 
				values (?, ?, ?, ?, ?)`,
				[thread_id, 'everyone', false, 'post', 'true']
			);
		} else {
			await db.query(
				`insert into permission (thread, type, mutable, permission, value) 
				values (?, ?, ?, ?, ?)`,
				[thread_id, 'members', false, 'post', 'true']
			);
		}
	} else {
		await db.query(
			`insert into permission (thread, type, mutable, permission, value) 
			values (?, ?, ?, ?, ?)`,
			[thread_id, 'members', false, 'view', 'true']
		);
		await db.query(
			`insert into permission (thread, type, mutable, permission, value) 
			values (?, ?, ?, ?, ?)`,
			[thread_id, 'members', false, 'post', 'true']
		);
	}
	// add members
	for (let user of msg.members) {
		if (!user) continue;
		// get user id
		const id = await db.query('select id from user where name = ?', [
			user.name,
		]);
		if (id.rows.length > 0) {
			const user_id = id.rows[0].id;
			await db.query(
				'insert into member (thread, user) values (?, ?)',
				[thread_id, user_id]
			);
		}
	}
	if (!msg.permissions || !msg.permissions.view_limited) {
		for (let username in vybe.users) {
			for (let socket of vybe.users[username].sockets) {
				socket.emit('new_thread', {
					name: msg.name,
					id: insert.rows[0].id,
					permissions: {
						is_member: false,
						view: true,
						post: !msg.permissions || !msg.permissions.post_limited
					}
				});
			}
		}
	}
	else {
		for (let member of msg.members) {
			for (let socket of vybe.users[member.name].sockets) {
				socket.emit('new_thread', {
					name: msg.name,
					id: insert.rows[0].id,
					permissions: {
						is_member: true,
						view: true,
						post: true
					}
				});
			}
		}
	}
	// respond
	return respond({
		success: true,
		id: insert.rows[0].id
	});
};

module.exports = authwrap(create_thread);
